export HeatEq
module HeatEq


using dg1d
using Plots


include("types.jl")
include("equation.jl")
include("rhs.jl")
include("initialdata.jl")
include("callbacks.jl")
include("setup.jl")


#######################################################################
#                 dg1d project interface definitions                  #
#######################################################################


dg1d.project_type(::Val{:HeatEq}) = Project

# TODO Remove bdryconds
dg1d.rhs!(env::Environment, P::Project, bdryconds) = rhs!(env, P, env.mesh)

dg1d.timestep!(env::Environment, P::Project) = timestep(env, P, env.mesh)

dg1d.@parameters HeatEq begin

  """
  Initial data configuration. Available options:
  - `sine`
  - `bump`
  - `gaussian`
  - `smoothed_bump`
  """
  id = "gaussian"
  @check id in [ "gaussian" ]

  "RMS width of gaussian"
  id_gaussian_width = 1.0

  "Amplitude of gaussian"
  id_gaussian_amplitude = 1.0

  """
  Boundary conditions. Available options:
  - `periodic`
  """
  bc = "periodic"
  @check bc in [ "periodic" ]

end


end # HeatEq
