function initialdata!(env, P::Project, prms)
  initialdata_equation!(env, env.mesh, P, P.equation, prms)
  initialdata_bdryconds!(env, env.mesh, P, P.bdrycond, prms)
  return
end


function initialdata_equation!(env, mesh::Mesh1d, P::Project, equation::AbstractHeatEq, prms)

  @unpack cache, mesh = env
  @unpack u = get_dynamic_variables(cache)
  @unpack L, x, xmin, xmax = mesh

  @unpack id = prms
  x = x[:]

  if id == "gaussian"
    A = prms["id_gaussian_amplitude"]
    w = prms["id_gaussian_width"]
    @toggled_assert w > 0.0
    x0 = (xmax + xmin) / 2
    u .= @. A * exp(-(x-x0)^2 / w^2)
  elseif id == "smooth bump"
    error("Unknown initialdata requested: '$id'")
  end

  return
end


function initialdata_bdryconds!(env, mesh, P, bdryconds, prms) end
function initialdata_bdryconds!(env, mesh::Mesh2d, P::Project, bdryconds, prms)
  @unpack init_u, init_flx_u, init_v,
          flx_u, q, flx_q, v = get_static_variables(mesh.cache)
  @unpack u = get_dynamic_variables(mesh.cache)
  init_u  .= u
  init_flx_u .= flx_u
  init_v  .= v
end
